'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _events = require('events');

var _lodash = require('lodash');

var _chat = require('./chat.js');

var _chat2 = _interopRequireDefault(_chat);

var _canvas = require('./canvas.js');

var _canvas2 = _interopRequireDefault(_canvas);

var _htmlContainer = require('./html-container');

var _htmlContainer2 = _interopRequireDefault(_htmlContainer);

var _clientStageManager = require('./client-stage-manager');

var _clientStageManager2 = _interopRequireDefault(_clientStageManager);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var builtinEvents = {
  _msg: function _msg(monsterr, msg) {
    monsterr.getChat().append(msg);
  }
};

var builtinClientOnlyEvents = {
  _start_stage: function _start_stage(monsterr, stageNo) {
    console.log('_start_stage', stageNo);
    monsterr.getStageManager().startStage(stageNo);
  },
  _end_stage: function _end_stage(monsterr, stageNo) {
    console.log('_end_stage', stageNo);
    monsterr.getStageManager().endStage(stageNo);
  },
  _game_over: function _game_over(monsterr) {
    console.log('_game_over');
  }
};

var builtinCommands = {
  clear: function clear(monsterr) {
    monsterr.getChat().clear();
    return false; // don't send this
  },
  id: function id(monsterr) {
    monsterr.getChat().append(monsterr.getId());
  }
};

function createClient() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options,
      _ref$events = _ref.events,
      events = _ref$events === undefined ? {} : _ref$events,
      _ref$commands = _ref.commands,
      commands = _ref$commands === undefined ? {} : _ref$commands,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages,
      _ref$adminClient = _ref.adminClient,
      adminClient = _ref$adminClient === undefined ? false : _ref$adminClient;

  var clientId = void 0;
  var emitter = new _events.EventEmitter();

  stages = (0, _lodash.flattenDeep)(stages);

  function sendCommand(type) {
    var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    emitter.emit('cmd', { type: type, args: args });
  }

  function sendEvent(type, payload) {
    emitter.emit('event', {
      type: type, payload: payload
    });
  }

  function log(msg, fileOrExtra, extra) {
    sendEvent('_log', { msg: msg, fileOrExtra: fileOrExtra, extra: extra });
  }

  var chat = (0, _chat2.default)({
    onCmd: function onCmd(cmd) {
      return monsterr.handleCommand(cmd);
    },
    onMsg: function onMsg(msg) {
      return sendEvent('_msg', msg);
    },
    hidden: options.hideChat || false
  });
  var htmlContainer = (0, _htmlContainer2.default)(options);
  var canvas = (0, _canvas2.default)(_extends({}, options, {
    getUsedWidth: function getUsedWidth() {
      return chat.isHidden() ? 0 : 300;
    },
    getUsedHeight: function getUsedHeight() {
      return htmlContainer.getHeightAbs();
    }
  }));

  var stageManager = !adminClient ? (0, _clientStageManager2.default)({
    stages: stages,
    getContext: function getContext() {
      return monsterr;
    },
    onStageFinished: function onStageFinished(stageNo) {
      monsterr.send('_stage_finished', stageNo);
    }
  }) : undefined;

  /** API */
  var monsterr = {
    send: sendEvent,
    sendCommand: sendCommand,
    log: log,

    setId: function setId(id) {
      clientId = id;
    },

    getId: function getId() {
      return clientId;
    },

    getChat: function getChat() {
      return chat;
    },
    getHtmlContainer: function getHtmlContainer() {
      return htmlContainer;
    },
    getCanvas: function getCanvas() {
      return canvas;
    },
    getStageManager: function getStageManager() {
      return stageManager;
    },

    handleEvent: function handleEvent(event) {
      return (0, _util.handleEvent)(event, [events, builtinEvents, adminClient ? {} : builtinClientOnlyEvents, adminClient ? {} : stageManager.getEvents()], monsterr);
    },
    handleCommand: function handleCommand(cmd) {
      var shouldCallServer = (0, _util.handleCommand)(cmd, [commands, builtinCommands, adminClient ? {} : stageManager.getCommands()], monsterr);

      if (shouldCallServer) {
        sendCommand(cmd.type, cmd.args);
      }
    },

    getCommands: function getCommands() {
      return commands;
    },
    getEvents: function getEvents() {
      return events;
    },
    getStages: function getStages() {
      return stages;
    },


    renderHtml: function renderHtml(html) {
      return htmlContainer.render(html);
    },
    disconnect: function disconnect() {
      htmlContainer.setHeightRatio(1);
      htmlContainer.render('<h1>Disconnected...</h1>');
    }
  };

  return { monsterr: monsterr, emitter: emitter };
}

exports.default = createClient;