'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _chat = require('./chat.js');

var _chat2 = _interopRequireDefault(_chat);

var _canvas = require('./canvas.js');

var _canvas2 = _interopRequireDefault(_canvas);

var _htmlContainer = require('./html-container');

var _htmlContainer2 = _interopRequireDefault(_htmlContainer);

var _clientStageManager = require('./client-stage-manager');

var _clientStageManager2 = _interopRequireDefault(_clientStageManager);

var _socketClient = require('./socket-client');

var _socketClient2 = _interopRequireDefault(_socketClient);

var _isAdmin = require('./is-admin');

var _isAdmin2 = _interopRequireDefault(_isAdmin);

var _util = require('../util');

var _events = require('./events');

var _commands = require('./commands');

var _events2 = require('../events');

var Events = _interopRequireWildcard(_events2);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createClient() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options,
      _ref$events = _ref.events,
      events = _ref$events === undefined ? {} : _ref$events,
      _ref$commands = _ref.commands,
      commands = _ref$commands === undefined ? {} : _ref$commands,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages;

  var clientId = void 0;
  var socketClient = (0, _socketClient2.default)(_isAdmin2.default ? '/admin' : '/clients');

  function log(msg, fileOrExtra, extra) {
    socketClient.sendEvent({
      type: Events.LOG,
      payload: { msg: msg, fileOrExtra: fileOrExtra, extra: extra }
    });
  }

  var chat = (0, _chat2.default)({
    onCmd: function onCmd(cmd) {
      var shouldCallServer = (0, _util.handleCommand)(cmd, [commands, _commands.builtinCommands, _isAdmin2.default ? {} : stageManager.getCommands()], monsterr);

      if (shouldCallServer) {
        socketClient.sendCommand(cmd);
      }
    },
    onMsg: function onMsg(msg) {
      return monsterr.send(Events.MESSAGE, msg);
    },
    hidden: options.hideChat || false
  });
  var htmlContainer = (0, _htmlContainer2.default)(options);
  var canvas = (0, _canvas2.default)(_extends({}, options, {
    getUsedWidth: function getUsedWidth() {
      return chat.isHidden() ? 0 : 300;
    },
    getUsedHeight: function getUsedHeight() {
      return htmlContainer.getHeightAbs();
    }
  }));

  var stageManager = !_isAdmin2.default ? (0, _clientStageManager2.default)({
    stages: stages,
    getContext: function getContext() {
      return monsterr;
    },
    onStageFinished: function onStageFinished(stageNo) {
      monsterr.send(Events.STAGE_FINISHED, stageNo);
    }
  }) : undefined;

  socketClient.on(Events.SET_ID, function (id) {
    clientId = id;
    chat.append('Your id is: ' + id);
  });
  socketClient.on(Events.CLIENT_DISCONNECTED, function () {
    return monsterr.disconnect();
  });
  socketClient.on('event', function (event) {
    (0, _util.handleEvent)(event, [events, _events.builtinEvents, _isAdmin2.default ? {} : _events.builtinClientEvents, _isAdmin2.default ? {} : stageManager.getEvents()], monsterr);
  });

  /** API */
  var monsterr = {
    send: function send(type, payload) {
      return socketClient.sendEvent({ type: type, payload: payload });
    },
    sendCommand: function sendCommand(type, args) {
      return socketClient.sendCommand({ type: type, args: args });
    },
    log: log,

    getId: function getId() {
      return clientId;
    },

    getChat: function getChat() {
      return chat;
    },
    getHtmlContainer: function getHtmlContainer() {
      return htmlContainer;
    },
    getCanvas: function getCanvas() {
      return canvas;
    },
    getStageManager: function getStageManager() {
      return stageManager;
    },

    renderHtml: function renderHtml(html) {
      return htmlContainer.render(html);
    },
    disconnect: function disconnect() {
      htmlContainer.setHeightRatio(1);
      htmlContainer.render('<h1>Disconnected...</h1>');
    },

    stageFinished: function stageFinished() {
      return stageManager.stageFinished();
    }
  };

  return monsterr;
}

exports.default = createClient;