'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createHttpServer;

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var app = (0, _express2.default)();
var httpServer = _http2.default.Server(app);
var io = (0, _socket2.default)(httpServer);

// Use sessions
var session = require('express-session')({
  secret: 'monstrous_secret',
  resave: true,
  saveUninitialized: true
});
var sharedSession = require('express-socket.io-session');
app.use(session);
io.of('/clients').use(sharedSession(session));

// We serve static from /assets. This allows games to easily include static assets by putting them in the /assets directory.
app.use(_express2.default.static('dist'));
app.use('/assets', _express2.default.static('assets'));

var hasPasswordMiddleware = function hasPasswordMiddleware(password) {
  return function (req, res, next) {
    if (!password) {
      return next();
    }

    if (req.query.key === password) {
      return next();
    }

    res.redirect('/');
  };
};

// Options are passed through to createServer inside of module
function createHttpServer(_ref) {
  var _ref$port = _ref.port,
      port = _ref$port === undefined ? 3000 : _ref$port,
      clientPassword = _ref.clientPassword,
      adminPassword = _ref.adminPassword;

  var clientMiddleware = hasPasswordMiddleware(clientPassword);
  var adminMiddleware = hasPasswordMiddleware(adminPassword);

  /* Routes */
  app.get('/admin', adminMiddleware, function (req, res) {
    return res.sendFile(_path2.default.join(__dirname, '../../index.html'));
  });
  app.get('/', clientMiddleware, function (req, res) {
    return res.sendFile(_path2.default.join(__dirname, '../../index.html'));
  });
  app.get('/fabric', function (req, res) {
    return res.sendFile(_path2.default.join(__dirname, '../../imports', 'fabric-2.2.3.js'));
  });

  httpServer.listen(port, function () {
    console.log('listening on ' + port);
  });

  return {
    getIO: function getIO() {
      return io;
    }
  };
}