'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = createSocketServer;

var _events = require('events');

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _events2 = require('../events');

var Events = _interopRequireWildcard(_events2);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SocketServer = function (_EventEmitter) {
  _inherits(SocketServer, _EventEmitter);

  function SocketServer(io) {
    _classCallCheck(this, SocketServer);

    var _this = _possibleConstructorReturn(this, (SocketServer.__proto__ || Object.getPrototypeOf(SocketServer)).call(this));

    _this.clientsNsp = io.of('/clients');
    _this.adminNsp = io.of('/admin');

    _this.socketMap = {};
    _this.latestHeartbeat = -1;
    _this.latencies = {};

    setInterval(function () {
      _this.sendHeartbeat();
    }, 5000);

    _this.setupClients();
    _this.setupAdmin();
    return _this;
  }

  _createClass(SocketServer, [{
    key: 'sendHeartbeat',
    value: function sendHeartbeat() {
      var _this2 = this;

      this.latestHeartbeat = Date.now();
      Object.entries(this.socketMap).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            uuid = _ref2[0],
            socket = _ref2[1];

        var latencies = _this2.latencies[uuid];
        var latest = latencies ? latencies.getLatest() : -1;
        var avg = latencies ? latencies.getAvg() : -1;
        socket.emit(Events.HEARTBEAT, { latest: latest, avg: avg });
      });
    }
  }, {
    key: 'handleHeartbeatAck',
    value: function handleHeartbeatAck(uuid) {
      var diff = Date.now() - this.latestHeartbeat;
      var latencies = this.latencies[uuid];
      if (!latencies) {
        this.latencies[uuid] = {
          data: [diff],
          getLatest: function getLatest() {
            return this.data[this.data.length - 1];
          },
          getAvg: function getAvg() {
            return Math.round(this.data.reduce(function (sum, e) {
              return sum + e;
            }, 0) / this.data.length);
          }
        };
      } else {
        latencies.data.push(diff);
        if (latencies.data.length >= 20) {
          latencies.data.shift();
        }
      }
    }
  }, {
    key: 'setupClients',
    value: function setupClients() {
      var _this3 = this;

      this.clientsNsp.on('connection', function (socket) {
        // Get (or generate id)
        var uuid = socket.handshake.session.uuid;
        if (!uuid) {
          socket.handshake.session.uuid = uuid = (0, _shortid2.default)();
          socket.handshake.session.save();
        }

        // new connection?
        if (!_this3.socketMap[uuid]) {
          _this3.emit(Events.CLIENT_CONNECTED, uuid);
        } else {
          _this3.socketMap[uuid].disconnect(); // disconnect prev socket
          _this3.emit(Events.CLIENT_RECONNECTED, uuid);
        }
        _this3.socketMap[uuid] = socket;

        // internal events
        socket.emit(Events.SET_ID, uuid);
        socket.on(Events.HEARTBEAT_ACK, function (_) {
          _this3.handleHeartbeatAck(uuid);
        });

        // Inbound
        var withClientId = function withClientId(cmdOrEvent) {
          return _extends({}, cmdOrEvent, { clientId: uuid });
        };
        socket.on('cmd', function (cmd) {
          return _this3.emit('cmd', withClientId(cmd));
        });
        socket.on('event', function (event) {
          return _this3.emit('event', withClientId(event));
        });

        socket.on('disconnect', function () {
          socket.removeAllListeners();
          setTimeout(function () {
            // if still dc'ed, disconnect
            if (_this3.socketMap[uuid].id === socket.id) {
              delete _this3.socketMap[uuid];
              _this3.emit(Events.CLIENT_DISCONNECTED, uuid);
            }
          }, 10000);
        });
      });
    }
  }, {
    key: 'setupAdmin',
    value: function setupAdmin() {
      var _this4 = this;

      this.adminNsp.on('connection', function (socket) {
        socket.on('cmd', function (cmd) {
          return _this4.emit('cmd', cmd);
        });
        socket.on('disconnect', function () {
          return socket.removeAllListeners();
        });
      });
    }
  }, {
    key: 'send',
    value: function send() {
      var _this5 = this;

      var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'event';
      var obj = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return {
        toAll: function toAll() {
          return _this5.clientsNsp.emit(type, obj);
        },
        toClients: function toClients() {
          var clients = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
          return clients.length && clients.map(function (uuid) {
            return _this5.socketMap[uuid].emit(type, obj);
          });
        },
        toAdmin: function toAdmin() {
          return _this5.adminNsp.emit(type, obj);
        }
      };
    }
  }, {
    key: 'sendCommand',
    value: function sendCommand(cmd) {
      return this.send('cmd', cmd);
    }
  }, {
    key: 'sendEvent',
    value: function sendEvent(event) {
      return this.send('event', event);
    }
  }, {
    key: 'getLatencies',
    value: function getLatencies() {
      return this.latencies;
    }
  }]);

  return SocketServer;
}(_events.EventEmitter);

function createSocketServer(io) {
  return new SocketServer(io);
}